/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.transfer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.xerces.dom.DocumentFragmentImpl;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.DocumentFragment;

import ch.cern.lhcb.xmleditor.xerces.CustomXMLSerializer;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * This defines a transferable data on top of TreeElement. This is used for drag
 * and drop of nodes
 * 
 * @author Sebastien Ponce
 */
public class TransferableNode implements Transferable {

    final static int STRING = 0;
    final static int PLAIN_TEXT = 1;
    final static int LOCAL = 2;

    /**
     * the dataflavors to transfer this class
     */
    public static final DataFlavor LocalFlavor = new DataFlavor(
            DataFlavor.javaJVMLocalObjectMimeType
                    + "; class=ch.cern.lhcb.xmleditor.xerces.TreeElement",
            "Local TreeElement");

    /**
     * the old deprecated plain 8-bit plain text flavor
     */
    public static final DataFlavor plainTextFlavor = new DataFlavor(
            "text/plain; charset=unicode", null);

    /**
     * the list of dataflavors provided
     */
    DataFlavor flavors[] = { DataFlavor.stringFlavor, plainTextFlavor,
            LocalFlavor};

    /**
     * the associated Nodes as a vector of TreeElements
     */
    private Vector nodes;

    /**
     * default constructor
     * 
     * @param nodes the TreeElements to be transfered, as a vector
     */
    public TransferableNode(Vector nodes) {
        // initializes the fields
        this.nodes = nodes;
    }

    /**
     * Returns an array of DataFlavor objects indicating the flavors the data
     * can be provided in. The array should be ordered according to preference
     * for providing the data (from most richly descriptive to least
     * descriptive).
     * 
     * @return an array of data flavors in which this data can be transferred
     */
    public synchronized DataFlavor[] getTransferDataFlavors() {
        return flavors;
    }

    /**
     * Returns whether or not the specified data flavor is supported for this
     * object.
     * 
     * @param flavor the requested flavor for the data
     * @return boolean indicating wjether or not the data flavor is supported
     */
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        return (flavor.equals(flavors[LOCAL])
                || flavor.equals(flavors[PLAIN_TEXT]) || flavor
                .equals(flavors[STRING]));
    }

    /**
     * gets a string representation of a given node (by serializing the node)
     * 
     * @param node the node to serialize
     * @return the xml code for Node node
     */
    private String getStringRepresentation(TreeElement node) {
        // creates an output stream
        OutputStream outputStream = new ByteArrayOutputStream();
        // defines the output format
        OutputFormat outputFormat = new OutputFormat(node.getOwnerDocument());
        outputFormat.setPreserveSpace(false);
        outputFormat.setIndenting(true);
        outputFormat.setLineWidth(0);
        outputFormat.setOmitDocumentType(true);
        outputFormat.setOmitXMLDeclaration(true);
        // creates a serializer
        XMLSerializer serializer = new CustomXMLSerializer(outputStream,
                outputFormat);
        // creates a documentfragment around the node to serialize
        DocumentFragment docFrag = new DocumentFragmentImpl((DocumentImpl) node
                .getOwnerDocument());
        // clone the node before using it to avoid it to be removed from its
        // parent
        docFrag.appendChild(node.cloneNode(true));
        // serializes the documentFragment
        try {
            serializer.serialize(docFrag);
        } catch (IOException e) {
            // will never be thrown
        }
        // returns the result
        return outputStream.toString();
    }

    /**
     * gets a string representation of this Transferable object (by serializing
     * eveery contained node, one by one)
     * 
     * @return the xml code corresponding to this Transferable Object
     */
    private String getStringRepresentation() {
        Enumeration enm = nodes.elements();
        String result = "";
        while (enm.hasMoreElements()) {
            result = result
                    + this.getStringRepresentation((TreeElement) enm
                            .nextElement());
        }
        return result;
    }

    /**
     * Returns an object which represents the data to be transferred. The class
     * of the object returned is defined by the representation class of the
     * flavor.
     * 
     * @param flavor the requested flavor for the data
     * @return an object which represents the data to be transferred
     * @throws IOException if the data is no longer available in the requested
     *         flavor.
     * @throws UnsupportedFlavorException if the requested data flavor is not
     *         supported.
     */
    public Object getTransferData(DataFlavor flavor)
            throws UnsupportedFlavorException, IOException {
        if (flavor.equals(LocalFlavor)) {
            // deals with the LocalFlavor, returns the node
            return nodes;
        } else if (flavor.equals(flavors[PLAIN_TEXT])) {
            return new StringReader(this.getStringRepresentation());
        } else if (flavor.equals(flavors[STRING])) {
            return this.getStringRepresentation();
        } else {
            // else raise an exception
            throw new UnsupportedFlavorException(flavor);
        }
    }

}