/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * BeanBuilder code Copyright (C) 2002 Mark Davidson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese.properties;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import com.simulogic.cese.ModelInfo;

/**
 * Table model used to obtain property names and values. This model encapsulates
 * an array of PropertyDescriptors.
 */
public class PropertyTableModel extends AbstractTableModel {

	private ModelPropertyDescriptor[] descriptors;

	private ModelInfo info;

	// Cached property editors.
	private Hashtable propEditors;

	// Shared instance of a comparator
	private static PropertyComparator comparator = new PropertyComparator();

	private static final int NUM_COLUMNS = 3;

	/**
	 * Index of name column
	 */
	public static final int COL_NAME = 0;

	/**
	 * Index of value column
	 */
	public static final int COL_VALUE = 1;

	/**
	 * Index of clamp column
	 */
	public static final int COL_CLAMP = 2;

	// View options
	public static final int VIEW_ALL = 0;

	public static final int VIEW_STANDARD = 1;

	public static final int VIEW_EXPERT = 2;

	public static final int VIEW_READ_ONLY = 3;

	public static final int VIEW_BOUND = 4;

	public static final int VIEW_CONSTRAINED = 5;

	public static final int VIEW_HIDDEN = 6;

	public static final int VIEW_PREFERRED = 7;

	// Sort options
	public static final int SORT_DEF = 0;

	public static final int SORT_NAME = 1;

	public static final int SORT_TYPE = 2;

	/**
	 * Default constructor
	 */
	public PropertyTableModel() {
		propEditors = new Hashtable();
	}

	/**
	 * Set the table model to represent the properties of the model.
	 * 
	 * @param info
	 *            model info
	 */
	public void setModelInfo(ModelInfo info) {
		this.info = info;
	}

	/**
	 * Get row count (total number of properties shown)
	 */
	public int getRowCount() {
		if (descriptors == null) {
			return 0;
		}

		return descriptors.length;
	}

	/**
	 * Get column count (3: name, value, clamp)
	 */
	public int getColumnCount() {
		return NUM_COLUMNS;
	}

	/**
	 * Check if given cell is editable
	 * 
	 * @param row
	 *            table row
	 * @param col
	 *            table column
	 */
	public boolean isCellEditable(int row, int col) {
		if ((col == COL_VALUE || col == COL_CLAMP) && (descriptors != null)) {
			return (descriptors[row].getDescriptor().getWriteMethod() == null) ? false
					: true;
		} else {
			return false;
		}
	}

	/**
	 * Get text value for cell of table
	 * 
	 * @param row
	 *            table row
	 * @param column
	 *            table column
	 */
	public Object getValueAt(int row, int column) {

		Object value = null;

		if (column == COL_NAME) {
			value = descriptors[row].getDescriptor().getDisplayName();
		}

		if (column == COL_VALUE) {
			Method getter = descriptors[row].getDescriptor().getReadMethod();

			if (getter != null) {
				Class[] paramTypes = getter.getParameterTypes();
				Object[] args = new Object[paramTypes.length];

				try {
					for (int i = 0; i < paramTypes.length; i++) {
						// XXX - debug
						System.out
								.println("Shouldn't happen! getValueAt getter = "
										+ getter
										+ " parameter = "
										+ paramTypes[i]);
						args[i] = paramTypes[i].newInstance();
					}
				} catch (Exception ex) {
					// XXX - handle better
					ex.printStackTrace();
				}

				try {
					value = getter.invoke(info.getModel(), args);
				} catch (IllegalArgumentException ex) {
					// XXX - handle better
					ex.printStackTrace();
				} catch (IllegalAccessException ex2) {
					// XXX - handle better
					ex2.printStackTrace();
				} catch (InvocationTargetException ex3) {
					// XXX - handle better
					ex3.printStackTrace();
				} catch (NoSuchMethodError ex4) {
					// XXX - handle better
					System.out
							.println("NoSuchMethodError for method invocation");
					System.out.println("Bean: " + info.getModel().toString());
					System.out.println("Getter: " + getter.getName());
					System.out.println("Getter args: ");
					for (int i = 0; i < args.length; i++) {
						System.out.println("\t" + "type: " + paramTypes[i]
								+ " value: " + args[i]);
					}
					ex4.printStackTrace();
				}
			}
		}

		if (column == COL_CLAMP) {
			value = new ClampAttribute(descriptors[row].getDescriptor()
					.getReadMethod().getName().substring(3), descriptors[row]
					.isClamp());
		}

		return value;
	}

	/**
	 * Set the value of the Values column.
	 */
	public void setValueAt(Object value, int row, int column) {

		if ((column == COL_NAME) || (descriptors == null)
				|| (row > descriptors.length)) {
			descriptors[row].setChanged(true);
			return;
		}

		if (column == COL_VALUE) {
			Method setter = descriptors[row].getDescriptor().getWriteMethod();
			if (setter != null) {
				try {
					setter.invoke(info.getModel(), new Object[] { value });
					info.setModified(true);
				} catch (IllegalArgumentException ex) {
					// XXX - handle better
					System.out.println("Setter: " + setter + "\nArgument: "
							+ value.getClass().toString());
					System.out.println("Row: " + row + " Column: " + column);
					ex.printStackTrace();
					System.out.println("\n");
				} catch (IllegalAccessException ex2) {
					// XXX - handle better
					System.out.println("Setter: " + setter + "\nArgument: "
							+ value.getClass().toString());
					System.out.println("Row: " + row + " Column: " + column);
					ex2.printStackTrace();
					System.out.println("\n");
				} catch (InvocationTargetException ex3) {
					// XXX - handle better
					System.out.println("Setter: " + setter + "\nArgument: "
							+ value.getClass().toString());
					System.out.println("Row: " + row + " Column: " + column);
					ex3.printStackTrace();
					System.out.println("\n");
				}
			}
		}

		if (column == COL_CLAMP) {
			descriptors[row].setClamp(((ClampAttribute) value).isClamped());
		}
	}

	/**
	 * Returns the Java type for the given column.
	 */
	public Class getColumnClass(int column) {
		if (column == COL_CLAMP)
			return Boolean.class;
		return null;
	}

	/**
	 * Returns the Java type info for the property at the given row
	 * 
	 * @param row
	 *            table row
	 * @return property type class
	 */
	public Class getPropertyType(int row) {
		return descriptors[row].getDescriptor().getPropertyType();
	}

	/**
	 * Returns the ModelPropertyDescriptor for the row
	 * 
	 * @param row
	 *            table row
	 * @return model property descriptor
	 */
	public ModelPropertyDescriptor getModelPropertyDescriptor(int row) {
		return descriptors[row];
	}

	/**
	 * Returns the Vector of PropertyDescriptors that are clamped
	 * 
	 * @return clamped propety descriptors
	 */
	public Vector getClampedPropertyDescriptors() {
		Vector r = new Vector();
		for (int i = 0; i < descriptors.length; i++) {
			if (descriptors[i].isClamp())
				r.addElement(descriptors[i].getDescriptor());
		}
		return r;
	}

	/**
	 * Clears changes in model property descriptors
	 */
	public void clearChanges () {
		for (int i = 0; i < descriptors.length; i++) {
			descriptors[i].setChanged(false);
		}
	}
	
	/**
	 * Adds changes to model property descriptors
	 */
	public void addChanges () {
		for (int i = 0; i < descriptors.length; i++) {
			descriptors[i].setValue(getValueAt(i,1));
		}
	}

	/**
	 * Sets clamped PropertyDescriptors based on the Vector of variable names
	 * 
	 * @param names
	 *            variable names
	 */
	public void setClampedPropertyDescriptors(Vector names) {
		for (int i = 0; i < descriptors.length; i++) {
			for (int j = 0; j < names.size(); j++) {
				if (descriptors[i].getDescriptor().getReadMethod().getName()
						.substring(3).equals(names.get(j)))
					setValueAt(new ClampAttribute((String) names.get(j), true),
							i, COL_CLAMP);
			}
		}
		fireTableDataChanged();
	}

	/**
	 * Returns a new instance of the property editor for a given class. If an
	 * editor is not specified in the property descriptor then it is looked up
	 * in the PropertyEditorManager.
	 * 
	 * @param row
	 *            table row
	 * @return property editor
	 */
	public PropertyEditor getPropertyEditor(int row) {
		Class cls = descriptors[row].getDescriptor().getPropertyEditorClass();

		PropertyEditor editor = null;

		if (cls != null) {
			try {
				editor = (PropertyEditor) cls.newInstance();
			} catch (Exception ex) {
				// XXX - debug
				System.out
						.println("PropertyTableModel: Instantiation exception creating PropertyEditor");
			}
		} else {
			// Look for a registered editor for this type.
			Class type = getPropertyType(row);
			if (type != null) {
				editor = (PropertyEditor) propEditors.get(type);

				if (editor == null) {
					// Load a shared instance of the property editor.
					editor = PropertyEditorManager.findEditor(type);
					if (editor != null)
						propEditors.put(type, editor);
				}

				if (editor == null) {
					// Use the editor for Object.class
					editor = (PropertyEditor) propEditors.get(Object.class);
					if (editor == null) {
						editor = PropertyEditorManager.findEditor(Object.class);
						if (editor != null)
							propEditors.put(Object.class, editor);
					}

				}
			}
		}
		return editor;
	}

	/**
	 * Sorts the table according to the sort type.
	 * 
	 * @param sort
	 *            sort type
	 */
	public void sortTable(int sort) {
		if (sort == SORT_DEF || descriptors == null)
			return;

		if (sort == SORT_NAME) {
			Arrays.sort(descriptors, comparator);
		} else {
			Arrays.sort(descriptors, comparator);
		}
		fireTableDataChanged();
	}

	/**
	 * Filters the table to display only properties with specific attributes.
	 * 
	 * @param view
	 *            The properties to display.
	 */
	public void filterTable(int view) {
		if (info == null)
			return;

		// Use collections to filter out unwanted properties
		ArrayList list = new ArrayList();
		list.addAll(Arrays.asList(info.getModelPropertyDescriptors()));

		ListIterator iterator = list.listIterator();
		ModelPropertyDescriptor desc;
		while (iterator.hasNext()) {
			desc = (ModelPropertyDescriptor) iterator.next();

			switch (view) {
			case VIEW_ALL:
				if (desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_STANDARD:
				if (desc.getDescriptor().getWriteMethod() == null
						|| desc.getDescriptor().isExpert()
						|| desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_EXPERT:
				if (desc.getDescriptor().getWriteMethod() == null
						|| !desc.getDescriptor().isExpert()
						|| desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_READ_ONLY:
				if (desc.getDescriptor().getWriteMethod() != null
						|| desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_HIDDEN:
				if (!desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_BOUND:
				if (!desc.getDescriptor().isBound()
						|| desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_CONSTRAINED:
				if (!desc.getDescriptor().isConstrained()
						|| desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			case VIEW_PREFERRED:
				if (!desc.getDescriptor().isPreferred()
						|| desc.getDescriptor().isHidden()) {
					iterator.remove();
				}
				break;
			}
		}

		descriptors = new ModelPropertyDescriptor[list.size()];
		for (int i = 0; i < descriptors.length; i++) {
			descriptors[i] = (ModelPropertyDescriptor) list.get(i);
		}

		sortTable(SORT_NAME);
	}

	/**
	 * Exports parameters to ASCII
	 * 
	 * @return String containing formatted ASCII codes for all the parameters
	 */
	public String exportParametersASCII() {
		StringBuffer datOut = new StringBuffer();
		String delimiter = "" + '\t';

		for (int i = 0; i < getRowCount(); i++) {
			for (int j = 0; j < getColumnCount(); j++) {
				datOut.append(getValueAt(i, j).toString() + delimiter);
			}
			datOut.append('\n');
		}

		return datOut.toString();
	}
}