/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.analysis;

import java.util.StringTokenizer;
import java.util.Vector;

/**
 * This class can format a list of selected indices (typically from the JList)
 * into the string
 * <p>
 * or parse a sweep selection string into the list of indices.
 */
public class SweepTokenizer {

    private static final String ddot = "..";
    private static final String comma = ",";

    /**
     * Parses the string containing sweep numbers into the list of indices to be
     * selected
     * <p>
     * Comma (",") separates sweep numbers and double dot ("..") sets interval
     * to be selected (both limits are included into the selection)
     * 
     * @param s string to be parsed
     * @return int[] array of selected sweep indices
     * @throws NumberFormatException if string can not be correctly parsed
     */
    public static final int[] parseSweeps(String s)
            throws NumberFormatException {

        Vector data = new Vector();
        String sn;

        s.trim();

        StringTokenizer st = new StringTokenizer(s, comma);

        while (st.hasMoreTokens()) {
            sn = st.nextToken().trim();
            try {
                data.add(new Integer(sn));
            } catch (NumberFormatException e) {
                int pos = sn.indexOf(ddot);
                if (pos == -1) { throw (new NumberFormatException()); }
                int beg = Integer.parseInt(sn.substring(0, pos).trim());
                int end = Integer.parseInt(sn.substring(pos + ddot.length(),
                        sn.length()).trim());

                if (beg > end) {
                    int swap = beg;
                    beg = end;
                    end = swap;
                }

                for (int i = beg; i <= end; i++) {
                    data.add(new Integer(i));
                }
            }
        }
        int[] indices = new int[data.size()];

        for (int i = 0; i < data.size(); i++) {
            indices[i] = ((Integer) data.get(i)).intValue();
        }

        return indices;
    }

    /**
     * Transforms the list of selected sweep indices into the string containing
     * sweep numbers
     * <p>
     * If the list contains 3 and more adjunct indices (e.g. 5,6,7..), they will
     * be converted into the interval form (5..7); otherwise, comma will be used
     * to separate sweep numbers
     * 
     * @param indices array of indices to process
     * @return String resulting string containing sweep numbers in the
     *         aforementioned form
     */
    public static final String parseSelection(int[] indices) {
        StringBuffer selection = new StringBuffer();

        Vector chunks = new Vector();

        int prev = -5;
        int delta = 0;
        boolean cont = false;

        for (int i = 0; i < indices.length; i++) {

            if ((indices[i] - prev) == 1) {
                delta++;
            } else {
                delta = 0;
                cont = false;
            }

            if (delta > 1) {
                if (!cont) {
                    chunks.remove(chunks.size() - 1);
                    chunks.remove(chunks.size() - 1);
                } else {
                    chunks.remove(chunks.size() - 1);
                }

                chunks.add(Integer.toString(indices[i] - delta) + ddot
                        + Integer.toString(indices[i]));
                cont = true;
            } else {
                chunks.add(Integer.toString(indices[i]));
            }

            prev = indices[i];
        }

        for (int i = 0; i < chunks.size(); i++) {
            selection.append((String) chunks.get(i));
            if (i < chunks.size() - 1) {
                selection.append(comma);
                selection.append(" ");
            }
        }
        return selection.toString();
    }

}