/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.plots;

import java.awt.BasicStroke;
import java.awt.Color;

/**
 * This class contains attributes that affect visual appearance of the channel
 */
public class ChannelAttribute {

    /**
     * Attribute of the normal (default) channel
     */
    public static final int NORMAL_CHANNEL = 0;

    /**
     * Attribute of the hidden channel
     */
    public static final int HIDDEN_CHANNEL = 1;

    /**
     * Default outline colors table
     */
    public static final Color outlineColors[] = { Color.green, Color.magenta,
            Color.cyan, Color.yellow, Color.pink, Color.blue, Color.red,
            Color.orange};

    /**
     * Default fill colors table
     */
    public static final Color fillColors[] = { Color.orange, Color.red,
            Color.blue, Color.pink, Color.yellow, Color.cyan, Color.magenta,
            Color.green};

    private Color color;
    private Color fillColor;
    private PlotSymbol symbol;
    private int visiblity;
    private double space = 0.5;
    private java.awt.BasicStroke stroke;

    private boolean selected;

    /**
     * Creates new ChannelAttributes
     */
    public ChannelAttribute() {
        this(Color.green, 0, 0);
    }

    /**
     * Creates new ChannelAttributes
     * 
     * @param color color of the channel
     * @param symbol symbol for the symbol plot
     * @param visiblity visiblity of the channel
     */
    public ChannelAttribute(Color color, int symbol, int visiblity) {
        this.visiblity = visiblity;
        this.color = color;

        this.stroke = new java.awt.BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);
        this.symbol = new PlotSymbol(0);

        this.selected = false;
    }

    /**
     * Returns true if this channel has normal (default) attribute turned on
     * 
     * @return true if channel is normal
     */
    public boolean isNormal() {
        return (visiblity == ChannelAttribute.NORMAL_CHANNEL);
    }

    /**
     * Returns true if this channel has hidden attribute turned on
     * 
     * @return true if channel is hidden
     */
    public boolean isHidden() {
        return (visiblity == ChannelAttribute.HIDDEN_CHANNEL);
    }

    /**
     * Sets color from the fixed outline color pallete
     * 
     * @param i index in the pallete
     */
    public void setIndexedColor(int i) {
        int c = i
                - (Math.round(i / outlineColors.length) * outlineColors.length);
        this.setColor(outlineColors[c]);
    }

    /**
     * Sets color from the fixed fill color pallete
     * 
     * @param i index in the pallete
     */
    public void setIndexedFillColor(int i) {
        int c = i - (Math.round(i / fillColors.length) * fillColors.length);
        this.setFillColor(fillColors[c]);
    }

    /**
     * Sets symbol from the fixed symbol pallete
     * 
     * @param i index in the pallete
     */
    public void setIndexedSymbol(int i) {
        int c = i
                - (Math.round(i / (PlotSymbol.symbols.length)) * (PlotSymbol.symbols.length));
        this.setSymbol(new PlotSymbol(c));
    }

    /**
     * Getter for property color
     * 
     * @return Value of property color
     */
    public Color getColor() {
        return color;
    }

    /**
     * Setter for property color
     * 
     * @param color New value of property color
     */
    public void setColor(Color color) {
        this.color = color;
    }

    /**
     * Getter for property fill color
     * 
     * @return Value of property fill color
     */
    public Color getFillColor() {
        return fillColor;
    }

    /**
     * Setter for property fill color
     * 
     * @param color New value of property fill color
     */
    public void setFillColor(Color color) {
        this.fillColor = color;
    }

    /**
     * Getter for property symbol.
     * 
     * @return Value of property symbol.
     */
    public PlotSymbol getSymbol() {
        return symbol;
    }

    /**
     * Setter for property symbol.
     * 
     * @param symbol New value of property symbol.
     */
    public void setSymbol(PlotSymbol symbol) {
        this.symbol = symbol;
    }

    /**
     * Getter for property space.
     * 
     * @return Value of property space.
     */
    public double getSpace() {
        return space;
    }

    /**
     * Setter for property space.
     * 
     * @param space New value of property space.
     */
    public void setSpace(double space) {
        this.space = space;
    }

    /**
     * Getter for property visiblity.
     * 
     * @return Value of property visiblity.
     */
    public int getVisiblity() {
        return visiblity;
    }

    /**
     * Setter for property visiblity.
     * 
     * @param visiblity New value of property visiblity.
     */
    public void setVisiblity(int visiblity) {
        this.visiblity = visiblity;
    }

    /**
     * Getter for property stroke.
     * 
     * @return Value of property stroke.
     */
    public java.awt.BasicStroke getStroke() {
        if (selected) return (new java.awt.BasicStroke(1.5f));
        return this.stroke;
    }

    /**
     * Setter for property stroke.
     * 
     * @param stroke New value of property stroke.
     */
    public void setStroke(java.awt.BasicStroke stroke) {
        this.stroke = stroke;
    }

    /**
     * Returns true if this channel is selected
     * 
     * @return true if channel is selected
     */
    public boolean isSelected() {
        return selected;
    }

    /**
     * Setter for property selected.
     * 
     * @param selected Value of property selected.
     */
    public void setSelected(boolean selected) {
        this.selected = selected;
    }
}