/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.specific;

import java.util.Enumeration;

import ch.cern.lhcb.xmleditor.Editor;
import ch.cern.lhcb.xmleditor.gui.MessageDisplayer;

/**
 * This class is a kind of accessor for dtd documents. It mainly implements a
 * cache on dtd documents.
 * 
 * @author Sebastien Ponce
 */
public class DtdFilesManager {

    /**
     * this pool keeps every file opened and parsed together with the number of
     * places where it is used.
     */
    private Pool refTable;

    /**
     * the editor using this Files Manager
     */
    private Editor editor;

    /**
     * Default constructor
     * 
     * @param editor the XmlEditor using this DtdFilesManager
     */
    public DtdFilesManager(Editor editor) {
        super();
        this.editor = editor;
        refTable = new Pool();
    }

    /**
     * gets a DTD from a file If this file was already accessed, just reuse it,
     * otherwise, open it, parse it and use it.
     * 
     * @param currentFile the current browsed file
     * @param fileName the name of the dtd file to be opened. Can be relative to
     *        the currentFile location
     * @param msgDis the MessageDisplayer to use for error displaying
     * @return the parsed file
     */
    public DtdFile getDtdFile(XmlFile currentFile, String fileName,
            MessageDisplayer msgDis) {
        // first parses the fileName
        DtdFile file = new DtdFile(currentFile, fileName, msgDis);
        // then call a private routine that does the job
        return getAndParseDocument(file, msgDis);
    }

    /**
     * gets a DTD from a file If this file was already accessed, just reuse it,
     * otherwise, open it, parse it and use it.
     * 
     * @param fileName the name of the dtd file to be opened.
     * @param msgDis the MessageDisplayer to use for error displaying
     * @return the parsed file
     */
    public DtdFile getDtdFile(String fileName, MessageDisplayer msgDis) {
        // first parses the fileName
        DtdFile file = new DtdFile(fileName, msgDis);
        // then call a private routine that does the job
        return getAndParseDocument(file, msgDis);
    }

    /**
     * gets a DTD from a file If this file was already accessed, just reuse it,
     * otherwise, open it, parse it and use it.
     * 
     * @param file the file to be opened
     * @param msgDis the MessageDisplayer used for error displaying
     * @return the parsed file
     */
    private DtdFile getAndParseDocument(DtdFile file, MessageDisplayer msgDis) {
        // true if the file we are looking for is already opened
        boolean alreadyOpened = false;
        // try to find the file in the list of already opened ones
        Enumeration enm = refTable.getPoolContent();
        while (enm.hasMoreElements()) {
            DtdFile openedFile = (DtdFile) enm.nextElement();
            if (file.equals(openedFile)) {
                file = openedFile;
                alreadyOpened = true;
                break;
            }
        }
        // if the file is not already opened, parse it and add it into the list
        if (!alreadyOpened) {
            if (!file.parse()) { return null; }
        }
        // adds a reference on the new file
        refTable.addRef(file);
        // finally returns the file
        return file;
    }

    /**
     * called when a file is closed. The file will only be closed if nobody else
     * uses it.
     * 
     * @param file the file to close
     */
    public void closeFile(DtdFile file) {
        refTable.removeRef(file);
        if (refTable.getRefNumber(file) == 0) {
            refTable.remove(file);
        }
    }

}